from typing import Any

class V1CustomResourceDefinitionNames:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, categories: list[str] | None = None, kind: str | None = None, list_kind: str | None = None, plural: str | None = None, short_names: list[str] | None = None, singular: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, categories: list[str]) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def list_kind(self) -> str: ...
    @list_kind.setter
    def list_kind(self, list_kind: str) -> None: ...
    @property
    def plural(self) -> str: ...
    @plural.setter
    def plural(self, plural: str) -> None: ...
    @property
    def short_names(self) -> list[str]: ...
    @short_names.setter
    def short_names(self, short_names: list[str]) -> None: ...
    @property
    def singular(self) -> str: ...
    @singular.setter
    def singular(self, singular: str) -> None: ...



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1CertificateSigningRequestCondition(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'last_transition_time': 'datetime',
        'last_update_time': 'datetime',
        'message': 'str',
        'reason': 'str',
        'status': 'str',
        'type': 'str'
    }

    attribute_map = {
        'last_transition_time': 'lastTransitionTime',
        'last_update_time': 'lastUpdateTime',
        'message': 'message',
        'reason': 'reason',
        'status': 'status',
        'type': 'type'
    }

    def __init__(self, last_transition_time=None, last_update_time=None, message=None, reason=None, status=None, type=None, local_vars_configuration=None):  # noqa: E501
        """V1CertificateSigningRequestCondition - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._last_transition_time = None
        self._last_update_time = None
        self._message = None
        self._reason = None
        self._status = None
        self._type = None
        self.discriminator = None

        if last_transition_time is not None:
            self.last_transition_time = last_transition_time
        if last_update_time is not None:
            self.last_update_time = last_update_time
        if message is not None:
            self.message = message
        if reason is not None:
            self.reason = reason
        self.status = status
        self.type = type

    @property
    def last_transition_time(self):
        """Gets the last_transition_time of this V1CertificateSigningRequestCondition.  # noqa: E501

        lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.  # noqa: E501

        :return: The last_transition_time of this V1CertificateSigningRequestCondition.  # noqa: E501
        :rtype: datetime
        """
        return self._last_transition_time

    @last_transition_time.setter
    def last_transition_time(self, last_transition_time):
        """Sets the last_transition_time of this V1CertificateSigningRequestCondition.

        lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.  # noqa: E501

        :param last_transition_time: The last_transition_time of this V1CertificateSigningRequestCondition.  # noqa: E501
        :type last_transition_time: datetime
        """

        self._last_transition_time = last_transition_time

    @property
    def last_update_time(self):
        """Gets the last_update_time of this V1CertificateSigningRequestCondition.  # noqa: E501

        lastUpdateTime is the time of the last update to this condition  # noqa: E501

        :return: The last_update_time of this V1CertificateSigningRequestCondition.  # noqa: E501
        :rtype: datetime
        """
        return self._last_update_time

    @last_update_time.setter
    def last_update_time(self, last_update_time):
        """Sets the last_update_time of this V1CertificateSigningRequestCondition.

        lastUpdateTime is the time of the last update to this condition  # noqa: E501

        :param last_update_time: The last_update_time of this V1CertificateSigningRequestCondition.  # noqa: E501
        :type last_update_time: datetime
        """

        self._last_update_time = last_update_time

    @property
    def message(self):
        """Gets the message of this V1CertificateSigningRequestCondition.  # noqa: E501

        message contains a human readable message with details about the request state  # noqa: E501

        :return: The message of this V1CertificateSigningRequestCondition.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this V1CertificateSigningRequestCondition.

        message contains a human readable message with details about the request state  # noqa: E501

        :param message: The message of this V1CertificateSigningRequestCondition.  # noqa: E501
        :type message: str
        """

        self._message = message

    @property
    def reason(self):
        """Gets the reason of this V1CertificateSigningRequestCondition.  # noqa: E501

        reason indicates a brief reason for the request state  # noqa: E501

        :return: The reason of this V1CertificateSigningRequestCondition.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this V1CertificateSigningRequestCondition.

        reason indicates a brief reason for the request state  # noqa: E501

        :param reason: The reason of this V1CertificateSigningRequestCondition.  # noqa: E501
        :type reason: str
        """

        self._reason = reason

    @property
    def status(self):
        """Gets the status of this V1CertificateSigningRequestCondition.  # noqa: E501

        status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be \"False\" or \"Unknown\".  # noqa: E501

        :return: The status of this V1CertificateSigningRequestCondition.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this V1CertificateSigningRequestCondition.

        status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be \"False\" or \"Unknown\".  # noqa: E501

        :param status: The status of this V1CertificateSigningRequestCondition.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def type(self):
        """Gets the type of this V1CertificateSigningRequestCondition.  # noqa: E501

        type of the condition. Known conditions are \"Approved\", \"Denied\", and \"Failed\".  An \"Approved\" condition is added via the /approval subresource, indicating the request was approved and should be issued by the signer.  A \"Denied\" condition is added via the /approval subresource, indicating the request was denied and should not be issued by the signer.  A \"Failed\" condition is added via the /status subresource, indicating the signer failed to issue the certificate.  Approved and Denied conditions are mutually exclusive. Approved, Denied, and Failed conditions cannot be removed once added.  Only one condition of a given type is allowed.  # noqa: E501

        :return: The type of this V1CertificateSigningRequestCondition.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this V1CertificateSigningRequestCondition.

        type of the condition. Known conditions are \"Approved\", \"Denied\", and \"Failed\".  An \"Approved\" condition is added via the /approval subresource, indicating the request was approved and should be issued by the signer.  A \"Denied\" condition is added via the /approval subresource, indicating the request was denied and should not be issued by the signer.  A \"Failed\" condition is added via the /status subresource, indicating the signer failed to issue the certificate.  Approved and Denied conditions are mutually exclusive. Approved, Denied, and Failed conditions cannot be removed once added.  Only one condition of a given type is allowed.  # noqa: E501

        :param type: The type of this V1CertificateSigningRequestCondition.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1CertificateSigningRequestCondition):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1CertificateSigningRequestCondition):
            return True

        return self.to_dict() != other.to_dict()



from typing import Any

class V1ContainerPort:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, container_port: int | None = None, host_ip: str | None = None, host_port: int | None = None, name: str | None = None, protocol: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def container_port(self) -> int: ...
    @container_port.setter
    def container_port(self, container_port: int) -> None: ...
    @property
    def host_ip(self) -> str: ...
    @host_ip.setter
    def host_ip(self, host_ip: str) -> None: ...
    @property
    def host_port(self) -> int: ...
    @host_port.setter
    def host_port(self, host_port: int) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def protocol(self) -> str: ...
    @protocol.setter
    def protocol(self, protocol: str) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1beta1IPAddress
from kubernetes_asyncio.client.models import V1beta1IPAddressList
from kubernetes_asyncio.client.models import V1beta1ServiceCIDR
from kubernetes_asyncio.client.models import V1beta1ServiceCIDRList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class NetworkingV1beta1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_ip_address(self, body: V1beta1IPAddress, **kwargs: Any) -> Awaitable[V1beta1IPAddress]: ...
    def create_ip_address_with_http_info(self, body: V1beta1IPAddress, **kwargs: Any) -> Awaitable[tuple[V1beta1IPAddress, int, CIMultiDictProxy]]: ...
    def create_service_cidr(self, body: V1beta1ServiceCIDR, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def create_service_cidr_with_http_info(self, body: V1beta1ServiceCIDR, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def delete_collection_ip_address(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_ip_address_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_service_cidr(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_service_cidr_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_ip_address(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_ip_address_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_service_cidr(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_service_cidr_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_ip_address(self, **kwargs: Any) -> Awaitable[V1beta1IPAddressList]: ...
    def list_ip_address_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta1IPAddressList, int, CIMultiDictProxy]]: ...
    def list_service_cidr(self, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDRList]: ...
    def list_service_cidr_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDRList, int, CIMultiDictProxy]]: ...
    def patch_ip_address(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta1IPAddress]: ...
    def patch_ip_address_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta1IPAddress, int, CIMultiDictProxy]]: ...
    def patch_service_cidr(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def patch_service_cidr_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def patch_service_cidr_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def patch_service_cidr_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def read_ip_address(self, name: str, **kwargs: Any) -> Awaitable[V1beta1IPAddress]: ...
    def read_ip_address_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1IPAddress, int, CIMultiDictProxy]]: ...
    def read_service_cidr(self, name: str, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def read_service_cidr_status(self, name: str, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def read_service_cidr_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def read_service_cidr_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def replace_ip_address(self, name: str, body: V1beta1IPAddress, **kwargs: Any) -> Awaitable[V1beta1IPAddress]: ...
    def replace_ip_address_with_http_info(self, name: str, body: V1beta1IPAddress, **kwargs: Any) -> Awaitable[tuple[V1beta1IPAddress, int, CIMultiDictProxy]]: ...
    def replace_service_cidr(self, name: str, body: V1beta1ServiceCIDR, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def replace_service_cidr_status(self, name: str, body: V1beta1ServiceCIDR, **kwargs: Any) -> Awaitable[V1beta1ServiceCIDR]: ...
    def replace_service_cidr_status_with_http_info(self, name: str, body: V1beta1ServiceCIDR, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def replace_service_cidr_with_http_info(self, name: str, body: V1beta1ServiceCIDR, **kwargs: Any) -> Awaitable[tuple[V1beta1ServiceCIDR, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1Affinity
from kubernetes_asyncio.client.models import V1Container
from kubernetes_asyncio.client.models import V1EphemeralContainer
from kubernetes_asyncio.client.models import V1HostAlias
from kubernetes_asyncio.client.models import V1LocalObjectReference
from kubernetes_asyncio.client.models import V1PodDNSConfig
from kubernetes_asyncio.client.models import V1PodOS
from kubernetes_asyncio.client.models import V1PodReadinessGate
from kubernetes_asyncio.client.models import V1PodResourceClaim
from kubernetes_asyncio.client.models import V1PodSchedulingGate
from kubernetes_asyncio.client.models import V1PodSecurityContext
from kubernetes_asyncio.client.models import V1ResourceRequirements
from kubernetes_asyncio.client.models import V1Toleration
from kubernetes_asyncio.client.models import V1TopologySpreadConstraint
from kubernetes_asyncio.client.models import V1Volume
from typing import Any

class V1PodSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, active_deadline_seconds: int | None = None, affinity: V1Affinity | None = None, automount_service_account_token: bool | None = None, containers: list[V1Container] | None = None, dns_config: V1PodDNSConfig | None = None, dns_policy: str | None = None, enable_service_links: bool | None = None, ephemeral_containers: list[V1EphemeralContainer] | None = None, host_aliases: list[V1HostAlias] | None = None, host_ipc: bool | None = None, host_network: bool | None = None, host_pid: bool | None = None, host_users: bool | None = None, hostname: str | None = None, hostname_override: str | None = None, image_pull_secrets: list[V1LocalObjectReference] | None = None, init_containers: list[V1Container] | None = None, node_name: str | None = None, node_selector: dict[str, str] | None = None, os: V1PodOS | None = None, overhead: dict[str, str] | None = None, preemption_policy: str | None = None, priority: int | None = None, priority_class_name: str | None = None, readiness_gates: list[V1PodReadinessGate] | None = None, resource_claims: list[V1PodResourceClaim] | None = None, resources: V1ResourceRequirements | None = None, restart_policy: str | None = None, runtime_class_name: str | None = None, scheduler_name: str | None = None, scheduling_gates: list[V1PodSchedulingGate] | None = None, security_context: V1PodSecurityContext | None = None, service_account: str | None = None, service_account_name: str | None = None, set_hostname_as_fqdn: bool | None = None, share_process_namespace: bool | None = None, subdomain: str | None = None, termination_grace_period_seconds: int | None = None, tolerations: list[V1Toleration] | None = None, topology_spread_constraints: list[V1TopologySpreadConstraint] | None = None, volumes: list[V1Volume] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def active_deadline_seconds(self) -> int: ...
    @active_deadline_seconds.setter
    def active_deadline_seconds(self, active_deadline_seconds: int) -> None: ...
    @property
    def affinity(self) -> V1Affinity: ...
    @affinity.setter
    def affinity(self, affinity: V1Affinity) -> None: ...
    @property
    def automount_service_account_token(self) -> bool: ...
    @automount_service_account_token.setter
    def automount_service_account_token(self, automount_service_account_token: bool) -> None: ...
    @property
    def containers(self) -> list[V1Container]: ...
    @containers.setter
    def containers(self, containers: list[V1Container]) -> None: ...
    @property
    def dns_config(self) -> V1PodDNSConfig: ...
    @dns_config.setter
    def dns_config(self, dns_config: V1PodDNSConfig) -> None: ...
    @property
    def dns_policy(self) -> str: ...
    @dns_policy.setter
    def dns_policy(self, dns_policy: str) -> None: ...
    @property
    def enable_service_links(self) -> bool: ...
    @enable_service_links.setter
    def enable_service_links(self, enable_service_links: bool) -> None: ...
    @property
    def ephemeral_containers(self) -> list[V1EphemeralContainer]: ...
    @ephemeral_containers.setter
    def ephemeral_containers(self, ephemeral_containers: list[V1EphemeralContainer]) -> None: ...
    @property
    def host_aliases(self) -> list[V1HostAlias]: ...
    @host_aliases.setter
    def host_aliases(self, host_aliases: list[V1HostAlias]) -> None: ...
    @property
    def host_ipc(self) -> bool: ...
    @host_ipc.setter
    def host_ipc(self, host_ipc: bool) -> None: ...
    @property
    def host_network(self) -> bool: ...
    @host_network.setter
    def host_network(self, host_network: bool) -> None: ...
    @property
    def host_pid(self) -> bool: ...
    @host_pid.setter
    def host_pid(self, host_pid: bool) -> None: ...
    @property
    def host_users(self) -> bool: ...
    @host_users.setter
    def host_users(self, host_users: bool) -> None: ...
    @property
    def hostname(self) -> str: ...
    @hostname.setter
    def hostname(self, hostname: str) -> None: ...
    @property
    def hostname_override(self) -> str: ...
    @hostname_override.setter
    def hostname_override(self, hostname_override: str) -> None: ...
    @property
    def image_pull_secrets(self) -> list[V1LocalObjectReference]: ...
    @image_pull_secrets.setter
    def image_pull_secrets(self, image_pull_secrets: list[V1LocalObjectReference]) -> None: ...
    @property
    def init_containers(self) -> list[V1Container]: ...
    @init_containers.setter
    def init_containers(self, init_containers: list[V1Container]) -> None: ...
    @property
    def node_name(self) -> str: ...
    @node_name.setter
    def node_name(self, node_name: str) -> None: ...
    @property
    def node_selector(self) -> dict[str, str]: ...
    @node_selector.setter
    def node_selector(self, node_selector: dict[str, str]) -> None: ...
    @property
    def os(self) -> V1PodOS: ...
    @os.setter
    def os(self, os: V1PodOS) -> None: ...
    @property
    def overhead(self) -> dict[str, str]: ...
    @overhead.setter
    def overhead(self, overhead: dict[str, str]) -> None: ...
    @property
    def preemption_policy(self) -> str: ...
    @preemption_policy.setter
    def preemption_policy(self, preemption_policy: str) -> None: ...
    @property
    def priority(self) -> int: ...
    @priority.setter
    def priority(self, priority: int) -> None: ...
    @property
    def priority_class_name(self) -> str: ...
    @priority_class_name.setter
    def priority_class_name(self, priority_class_name: str) -> None: ...
    @property
    def readiness_gates(self) -> list[V1PodReadinessGate]: ...
    @readiness_gates.setter
    def readiness_gates(self, readiness_gates: list[V1PodReadinessGate]) -> None: ...
    @property
    def resource_claims(self) -> list[V1PodResourceClaim]: ...
    @resource_claims.setter
    def resource_claims(self, resource_claims: list[V1PodResourceClaim]) -> None: ...
    @property
    def resources(self) -> V1ResourceRequirements: ...
    @resources.setter
    def resources(self, resources: V1ResourceRequirements) -> None: ...
    @property
    def restart_policy(self) -> str: ...
    @restart_policy.setter
    def restart_policy(self, restart_policy: str) -> None: ...
    @property
    def runtime_class_name(self) -> str: ...
    @runtime_class_name.setter
    def runtime_class_name(self, runtime_class_name: str) -> None: ...
    @property
    def scheduler_name(self) -> str: ...
    @scheduler_name.setter
    def scheduler_name(self, scheduler_name: str) -> None: ...
    @property
    def scheduling_gates(self) -> list[V1PodSchedulingGate]: ...
    @scheduling_gates.setter
    def scheduling_gates(self, scheduling_gates: list[V1PodSchedulingGate]) -> None: ...
    @property
    def security_context(self) -> V1PodSecurityContext: ...
    @security_context.setter
    def security_context(self, security_context: V1PodSecurityContext) -> None: ...
    @property
    def service_account(self) -> str: ...
    @service_account.setter
    def service_account(self, service_account: str) -> None: ...
    @property
    def service_account_name(self) -> str: ...
    @service_account_name.setter
    def service_account_name(self, service_account_name: str) -> None: ...
    @property
    def set_hostname_as_fqdn(self) -> bool: ...
    @set_hostname_as_fqdn.setter
    def set_hostname_as_fqdn(self, set_hostname_as_fqdn: bool) -> None: ...
    @property
    def share_process_namespace(self) -> bool: ...
    @share_process_namespace.setter
    def share_process_namespace(self, share_process_namespace: bool) -> None: ...
    @property
    def subdomain(self) -> str: ...
    @subdomain.setter
    def subdomain(self, subdomain: str) -> None: ...
    @property
    def termination_grace_period_seconds(self) -> int: ...
    @termination_grace_period_seconds.setter
    def termination_grace_period_seconds(self, termination_grace_period_seconds: int) -> None: ...
    @property
    def tolerations(self) -> list[V1Toleration]: ...
    @tolerations.setter
    def tolerations(self, tolerations: list[V1Toleration]) -> None: ...
    @property
    def topology_spread_constraints(self) -> list[V1TopologySpreadConstraint]: ...
    @topology_spread_constraints.setter
    def topology_spread_constraints(self, topology_spread_constraints: list[V1TopologySpreadConstraint]) -> None: ...
    @property
    def volumes(self) -> list[V1Volume]: ...
    @volumes.setter
    def volumes(self, volumes: list[V1Volume]) -> None: ...



from kubernetes_asyncio.client.models import V1GroupSubject
from kubernetes_asyncio.client.models import V1ServiceAccountSubject
from kubernetes_asyncio.client.models import V1UserSubject
from typing import Any

class FlowcontrolV1Subject:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, group: V1GroupSubject | None = None, kind: str | None = None, service_account: V1ServiceAccountSubject | None = None, user: V1UserSubject | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def group(self) -> V1GroupSubject: ...
    @group.setter
    def group(self, group: V1GroupSubject) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def service_account(self) -> V1ServiceAccountSubject: ...
    @service_account.setter
    def service_account(self, service_account: V1ServiceAccountSubject) -> None: ...
    @property
    def user(self) -> V1UserSubject: ...
    @user.setter
    def user(self, user: V1UserSubject) -> None: ...



from kubernetes_asyncio.client.models import V2MetricValueStatus
from typing import Any

class V2ContainerResourceMetricStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, container: str | None = None, current: V2MetricValueStatus | None = None, name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def container(self) -> str: ...
    @container.setter
    def container(self, container: str) -> None: ...
    @property
    def current(self) -> V2MetricValueStatus: ...
    @current.setter
    def current(self, current: V2MetricValueStatus) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...



from kubernetes_asyncio.client.models import V1LoadBalancerIngress
from typing import Any

class V1LoadBalancerStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, ingress: list[V1LoadBalancerIngress] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def ingress(self) -> list[V1LoadBalancerIngress]: ...
    @ingress.setter
    def ingress(self, ingress: list[V1LoadBalancerIngress]) -> None: ...



from kubernetes_asyncio.client.models import V1CustomResourceDefinitionCondition
from kubernetes_asyncio.client.models import V1CustomResourceDefinitionNames
from typing import Any

class V1CustomResourceDefinitionStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, accepted_names: V1CustomResourceDefinitionNames | None = None, conditions: list[V1CustomResourceDefinitionCondition] | None = None, stored_versions: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def accepted_names(self) -> V1CustomResourceDefinitionNames: ...
    @accepted_names.setter
    def accepted_names(self, accepted_names: V1CustomResourceDefinitionNames) -> None: ...
    @property
    def conditions(self) -> list[V1CustomResourceDefinitionCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1CustomResourceDefinitionCondition]) -> None: ...
    @property
    def stored_versions(self) -> list[str]: ...
    @stored_versions.setter
    def stored_versions(self, stored_versions: list[str]) -> None: ...



from kubernetes_asyncio.client.models import V2ContainerResourceMetricSource
from kubernetes_asyncio.client.models import V2ExternalMetricSource
from kubernetes_asyncio.client.models import V2ObjectMetricSource
from kubernetes_asyncio.client.models import V2PodsMetricSource
from kubernetes_asyncio.client.models import V2ResourceMetricSource
from typing import Any

class V2MetricSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, container_resource: V2ContainerResourceMetricSource | None = None, external: V2ExternalMetricSource | None = None, object: V2ObjectMetricSource | None = None, pods: V2PodsMetricSource | None = None, resource: V2ResourceMetricSource | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def container_resource(self) -> V2ContainerResourceMetricSource: ...
    @container_resource.setter
    def container_resource(self, container_resource: V2ContainerResourceMetricSource) -> None: ...
    @property
    def external(self) -> V2ExternalMetricSource: ...
    @external.setter
    def external(self, external: V2ExternalMetricSource) -> None: ...
    @property
    def object(self) -> V2ObjectMetricSource: ...
    @object.setter
    def object(self, object: V2ObjectMetricSource) -> None: ...
    @property
    def pods(self) -> V2PodsMetricSource: ...
    @pods.setter
    def pods(self, pods: V2PodsMetricSource) -> None: ...
    @property
    def resource(self) -> V2ResourceMetricSource: ...
    @resource.setter
    def resource(self, resource: V2ResourceMetricSource) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V1Condition
from kubernetes_asyncio.client.models import V1TypeChecking
from typing import Any

class V1ValidatingAdmissionPolicyStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, conditions: list[V1Condition] | None = None, observed_generation: int | None = None, type_checking: V1TypeChecking | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def conditions(self) -> list[V1Condition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1Condition]) -> None: ...
    @property
    def observed_generation(self) -> int: ...
    @observed_generation.setter
    def observed_generation(self, observed_generation: int) -> None: ...
    @property
    def type_checking(self) -> V1TypeChecking: ...
    @type_checking.setter
    def type_checking(self, type_checking: V1TypeChecking) -> None: ...



from kubernetes_asyncio.client.models import V1GroupVersionForDiscovery
from kubernetes_asyncio.client.models import V1ServerAddressByClientCIDR
from typing import Any

class V1APIGroup:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, api_version: str | None = None, kind: str | None = None, name: str | None = None, preferred_version: V1GroupVersionForDiscovery | None = None, server_address_by_client_cidrs: list[V1ServerAddressByClientCIDR] | None = None, versions: list[V1GroupVersionForDiscovery] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def api_version(self) -> str: ...
    @api_version.setter
    def api_version(self, api_version: str) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def preferred_version(self) -> V1GroupVersionForDiscovery: ...
    @preferred_version.setter
    def preferred_version(self, preferred_version: V1GroupVersionForDiscovery) -> None: ...
    @property
    def server_address_by_client_cidrs(self) -> list[V1ServerAddressByClientCIDR]: ...
    @server_address_by_client_cidrs.setter
    def server_address_by_client_cidrs(self, server_address_by_client_cidrs: list[V1ServerAddressByClientCIDR]) -> None: ...
    @property
    def versions(self) -> list[V1GroupVersionForDiscovery]: ...
    @versions.setter
    def versions(self, versions: list[V1GroupVersionForDiscovery]) -> None: ...



from kubernetes_asyncio.client.models import V1beta1BasicDevice
from typing import Any

class V1beta1Device:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, basic: V1beta1BasicDevice | None = None, name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def basic(self) -> V1beta1BasicDevice: ...
    @basic.setter
    def basic(self, basic: V1beta1BasicDevice) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...



# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
from functools import partial
from pprint import pformat
from typing import TYPE_CHECKING, Any, Awaitable, Callable

import yaml

if TYPE_CHECKING:
    from kubernetes_asyncio.dynamic.client import DynamicClient


class Resource:
    """Represents an API resource type, containing the information required to build urls for requests"""

    def __init__(
        self,
        prefix: str | None = None,
        group: str | None = None,
        api_version: str | None = None,
        kind: str | None = None,
        namespaced: bool = False,
        verbs: str | None = None,
        name: str | None = None,
        preferred: bool = False,
        client: "DynamicClient | None" = None,
        singular_name: str | None = None,
        short_names: str | None = None,
        categories: str | None = None,
        subresources: dict | None = None,
        **kwargs,
    ):
        if None in (api_version, kind, prefix):
            raise ValueError("At least prefix, kind, and api_version must be provided")

        self.prefix = prefix
        self.group = group
        self.api_version = api_version
        self.kind = kind
        self.namespaced = namespaced
        self.verbs = verbs
        self.name = name
        self.preferred = preferred
        self.client = client
        self.singular_name = singular_name or (name[:-1] if name else "")
        self.short_names = short_names
        self.categories = categories
        self.subresources = {
            k: Subresource(self, **v) for k, v in (subresources or {}).items()
        }

        self.extra_args = kwargs

    def to_dict(self) -> dict[str, Any]:
        d = {
            "_type": "Resource",
            "prefix": self.prefix,
            "group": self.group,
            "api_version": self.api_version,
            "kind": self.kind,
            "namespaced": self.namespaced,
            "verbs": self.verbs,
            "name": self.name,
            "preferred": self.preferred,
            "singularName": self.singular_name,
            "shortNames": self.short_names,
            "categories": self.categories,
            "subresources": {k: sr.to_dict() for k, sr in self.subresources.items()},
        }
        d.update(self.extra_args)
        return d

    @property
    def group_version(self) -> str | None:
        if self.group:
            return f"{self.group}/{self.api_version}"
        return self.api_version

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}({self.group_version}/{self.name})>"

    @property
    def urls(self) -> dict[str, str]:
        full_prefix = f"{self.prefix}/{self.group_version}"
        resource_name = self.name.lower() if self.name else ""
        return {
            "base": "/{}/{}".format(full_prefix, resource_name),
            "namespaced_base": "/{}/namespaces/{{namespace}}/{}".format(
                full_prefix, resource_name
            ),
            "full": "/{}/{}/{{name}}".format(full_prefix, resource_name),
            "namespaced_full": "/{}/namespaces/{{namespace}}/{}/{{name}}".format(
                full_prefix, resource_name
            ),
        }

    def path(self, name: str | None = None, namespace: str | None = None) -> str:
        url_type = []
        path_params = {}
        if self.namespaced and namespace:
            url_type.append("namespaced")
            path_params["namespace"] = namespace
        if name:
            url_type.append("full")
            path_params["name"] = name
        else:
            url_type.append("base")
        return self.urls["_".join(url_type)].format(**path_params)

    def __getattr__(self, name: str) -> Callable[..., "Awaitable[ResourceInstance]"]:
        if name in self.subresources:
            return self.subresources[name]  # type: ignore
        return partial(getattr(self.client, name), self)


class ResourceList(Resource):
    """Represents a list of API objects"""

    def __init__(
        self,
        client: "DynamicClient",
        group: str | None = None,
        api_version: str | None = None,
        base_kind: str | None = None,
        kind: str | None = None,
        base_resource_lookup=None,
    ):
        self.client = client
        self.group = group or ""
        self.api_version = api_version or "v1"
        self.kind = kind or "{}List".format(base_kind)
        self.base_kind = base_kind or ""
        self.base_resource_lookup = base_resource_lookup
        self.__base_resource = None

    async def _ainit(self):
        if self.base_kind and self.api_version and self.group:
            self.__base_resource = await self.client.resources.get(
                group=self.group, api_version=self.api_version, kind=self.base_kind
            )

    @classmethod
    async def create_list(
        cls, client, group="", api_version="v1", base_kind="", kind=None
    ):
        self = cls(
            client=client,
            group=group,
            api_version=api_version,
            base_kind=base_kind,
            kind=kind,
        )
        await self._ainit()
        return self

    # TODO: This code appears to be unused, or at least untested
    async def base_resource(self):
        if self.__base_resource:
            return self.__base_resource
        elif self.base_resource_lookup:
            self.__base_resource = await self.client.resources.get(
                **self.base_resource_lookup
            )
            return self.__base_resource
        elif self.base_kind:
            self.__base_resource = await self.client.resources.get(
                group=self.group, api_version=self.api_version, kind=self.base_kind
            )
            return self.__base_resource
        return None

    # TODO: This code appears to be unused, or at least untested
    async def _items_to_resources(self, body):
        """Takes a List body and return a dictionary with the following structure:
        {
            'api_version': str,
            'kind': str,
            'items': [{
                'resource': Resource,
                'name': str,
                'namespace': str,
            }]
        }
        """
        if body is None:
            raise ValueError(
                "You must provide a body when calling methods on a ResourceList"
            )

        api_version = body["apiVersion"]
        kind = body["kind"]
        items = body.get("items")
        if not items:
            raise ValueError(
                "The `items` field in the body must be populated when calling methods on a ResourceList"
            )

        if self.kind != kind:
            raise ValueError(
                f"Methods on a {self.kind} must be called with a body containing the same kind."
                f" Received {kind} instead"
            )

        return {
            "api_version": api_version,
            "kind": kind,
            "items": [await self._item_to_resource(item) for item in items],
        }

    # TODO: This code appears to be unused, or at least untested
    async def _item_to_resource(self, item):
        metadata = item.get("metadata", {})
        resource = await self.base_resource()
        if not resource:
            api_version = item.get("apiVersion", self.api_version)
            kind = item.get("kind", self.base_kind)
            resource = await self.client.resources.get(
                api_version=api_version, kind=kind
            )
        return {
            "resource": resource,
            "definition": item,
            "name": metadata.get("name"),
            "namespace": metadata.get("namespace"),
        }

    async def get(self, body, name=None, namespace=None, **kwargs):
        if name:
            raise ValueError(
                "Operations on ResourceList objects do not support the `name` argument"
            )
        resource_list = await self._items_to_resources(body)
        response = copy.deepcopy(body)

        response["items"] = [
            item["resource"]
            .get(name=item["name"], namespace=item["namespace"] or namespace, **kwargs)
            .to_dict()
            for item in resource_list["items"]
        ]
        return ResourceInstance(self, response)

    async def delete(self, body, name=None, namespace=None, **kwargs):
        if name:
            raise ValueError(
                "Operations on ResourceList objects do not support the `name` argument"
            )
        resource_list = await self._items_to_resources(body)
        response = copy.deepcopy(body)

        response["items"] = [
            item["resource"]
            .delete(
                name=item["name"], namespace=item["namespace"] or namespace, **kwargs
            )
            .to_dict()
            for item in resource_list["items"]
        ]
        return ResourceInstance(self, response)

    async def verb_mapper(self, verb, body, **kwargs):
        resource_list = await self._items_to_resources(body)
        response = copy.deepcopy(body)
        response["items"] = [
            getattr(item["resource"], verb)(body=item["definition"], **kwargs).to_dict()
            for item in resource_list["items"]
        ]
        return ResourceInstance(self, response)

    async def create(self, *args, **kwargs):
        return await self.verb_mapper("create", *args, **kwargs)

    async def replace(self, *args, **kwargs):
        return await self.verb_mapper("replace", *args, **kwargs)

    async def patch(self, *args, **kwargs):
        return await self.verb_mapper("patch", *args, **kwargs)

    def to_dict(self):
        return {
            "_type": "ResourceList",
            "group": self.group,
            "api_version": self.api_version,
            "kind": self.kind,
            "base_kind": self.base_kind,
        }

    # This code is not executed in any test scenario - is it needed?
    def __getattr__(self, name: str) -> Any:
        if self.base_resource():
            return getattr(self.base_resource(), name)
        return None


class Subresource(Resource):
    """Represents a subresource of an API resource. This generally includes operations
    like scale, as well as status objects for an instantiated resource
    """

    def __init__(self, parent, **kwargs):  # noqa
        # super().__init__()
        self.parent = parent
        self.prefix = parent.prefix
        self.group = parent.group
        self.api_version = parent.api_version
        self.kind = kwargs.pop("kind")
        self.name = kwargs.pop("name")
        self.subresource = kwargs.pop("subresource", None) or self.name.split("/")[1]
        self.namespaced = kwargs.pop("namespaced", False)
        self.verbs = kwargs.pop("verbs", None)
        self.extra_args = kwargs

    # TODO(fabianvf): Determine proper way to handle differences between resources + subresources
    async def create(
        self,
        body=None,
        name: str | None = None,
        namespace: str | None = None,
        **kwargs: Any,
    ):
        name = name
        if name is None:
            name = body.get("metadata", {}).get("name") if body else None
        body = self.parent.client.serialize_body(body)
        if self.parent.namespaced:
            namespace = self.parent.client.ensure_namespace(
                self.parent, namespace, body
            )
        path = self.path(name=name, namespace=namespace)
        return await self.parent.client.request("post", path, body=body, **kwargs)

    @property
    def urls(self) -> dict[str, str]:
        full_prefix = f"{self.prefix}/{self.group_version}"
        return {
            "full": "/{}/{}/{{name}}/{}".format(
                full_prefix, self.parent.name, self.subresource
            ),
            "namespaced_full": "/{}/namespaces/{{namespace}}/{}/{{name}}/{}".format(
                full_prefix, self.parent.name, self.subresource
            ),
        }

    def __getattr__(self, name):
        return partial(getattr(self.parent.client, name), self)

    def to_dict(self) -> dict[str, str]:
        d = {
            "kind": self.kind,
            "name": self.name,
            "subresource": self.subresource,
            "namespaced": self.namespaced,
            "verbs": self.verbs,
        }
        d.update(self.extra_args)
        return d


class ResourceInstance:
    """A parsed instance of an API resource. It exists solely to
    ease interaction with API objects by allowing attributes to
    be accessed with '.' notation.
    """

    def __init__(self, client, instance):
        self.client = client
        # If we have a list of resources, then set the apiVersion and kind of
        # each resource in 'items'
        kind = instance["kind"]
        if kind.endswith("List") and "items" in instance:
            kind = instance["kind"][:-4]
            if instance['items'] is None:
                instance['items'] = []
            for item in instance["items"]:
                if "apiVersion" not in item:
                    item["apiVersion"] = instance["apiVersion"]
                if "kind" not in item:
                    item["kind"] = kind

        self.attributes = self.__deserialize(instance)
        self.__initialised = True

    def __deserialize(self, field):
        if isinstance(field, dict):
            return ResourceField(
                params={k: self.__deserialize(v) for k, v in field.items()}
            )
        elif isinstance(field, (list, tuple)):
            return [self.__deserialize(item) for item in field]
        else:
            return field

    def __serialize(self, field):
        if isinstance(field, ResourceField):
            return {k: self.__serialize(v) for k, v in field.__dict__.items()}
        elif isinstance(field, (list, tuple)):
            return [self.__serialize(item) for item in field]
        elif isinstance(field, ResourceInstance):
            return field.to_dict()
        else:
            return field

    def to_dict(self):
        return self.__serialize(self.attributes)

    def to_str(self):
        return repr(self)

    def __repr__(self):
        return "ResourceInstance[{}]:\n  {}".format(
            self.attributes.kind,
            "  ".join(yaml.safe_dump(self.to_dict()).splitlines(True)),
        )

    def __getattr__(self, name):
        if "_ResourceInstance__initialised" not in self.__dict__:
            return super().__getattr__(name)  # type: ignore
        return getattr(self.attributes, name)

    def __setattr__(self, name, value):
        if "_ResourceInstance__initialised" not in self.__dict__:
            return super().__setattr__(name, value)
        elif name in self.__dict__:
            return super().__setattr__(name, value)
        else:
            self.attributes[name] = value

    def __getitem__(self, name):
        return self.attributes[name]

    def __setitem__(self, name, value):
        self.attributes[name] = value

    def __dir__(self):
        return dir(type(self)) + list(self.attributes.__dict__.keys())


class ResourceField(object):
    """A parsed instance of an API resource attribute. It exists
    solely to ease interaction with API objects by allowing
    attributes to be accessed with '.' notation
    """

    def __init__(self, params):
        self.__dict__.update(**params)

    def __repr__(self):
        return pformat(self.__dict__)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __getitem__(self, name: str):
        return self.__dict__.get(name)

    # Here resource.items will return items if available or resource.__dict__.items function if not
    # resource.get will call resource.__dict__.get after attempting resource.__dict__.get('get')
    def __getattr__(self, name):
        return self.__dict__.get(name, getattr(self.__dict__, name, None))

    def __setattr__(self, name, value):
        self.__dict__[name] = value

    def __dir__(self):
        return dir(type(self)) + list(self.__dict__.keys())

    def __iter__(self):
        for k, v in self.__dict__.items():
            yield (k, v)

    def to_dict(self) -> dict:
        ret = self.__serialize(self)
        if isinstance(ret, dict):
            return ret
        return {"to_dict": ret}

    def __serialize(self, field) -> dict | list:
        if isinstance(field, ResourceField):
            return {k: self.__serialize(v) for k, v in field.__dict__.items()}
        if isinstance(field, (list, tuple)):
            return [self.__serialize(item) for item in field]
        return field



from kubernetes_asyncio.client.models import V1NodeConfigSource
from kubernetes_asyncio.client.models import V1Taint
from typing import Any

class V1NodeSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, config_source: V1NodeConfigSource | None = None, external_id: str | None = None, pod_cidr: str | None = None, pod_cidrs: list[str] | None = None, provider_id: str | None = None, taints: list[V1Taint] | None = None, unschedulable: bool | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def config_source(self) -> V1NodeConfigSource: ...
    @config_source.setter
    def config_source(self, config_source: V1NodeConfigSource) -> None: ...
    @property
    def external_id(self) -> str: ...
    @external_id.setter
    def external_id(self, external_id: str) -> None: ...
    @property
    def pod_cidr(self) -> str: ...
    @pod_cidr.setter
    def pod_cidr(self, pod_cidr: str) -> None: ...
    @property
    def pod_cidrs(self) -> list[str]: ...
    @pod_cidrs.setter
    def pod_cidrs(self, pod_cidrs: list[str]) -> None: ...
    @property
    def provider_id(self) -> str: ...
    @provider_id.setter
    def provider_id(self, provider_id: str) -> None: ...
    @property
    def taints(self) -> list[V1Taint]: ...
    @taints.setter
    def taints(self, taints: list[V1Taint]) -> None: ...
    @property
    def unschedulable(self) -> bool: ...
    @unschedulable.setter
    def unschedulable(self, unschedulable: bool) -> None: ...



# Copyright 2016 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import os

from kubernetes_asyncio.client import Configuration
from kubernetes_asyncio.config.config_exception import ConfigException

SERVICE_HOST_ENV_NAME = "KUBERNETES_SERVICE_HOST"
SERVICE_PORT_ENV_NAME = "KUBERNETES_SERVICE_PORT"
SERVICE_TOKEN_FILENAME = "/var/run/secrets/kubernetes.io/serviceaccount/token"
SERVICE_CERT_FILENAME = "/var/run/secrets/kubernetes.io/serviceaccount/ca.crt"

TOKEN_REFRESH_PERIOD = datetime.timedelta(minutes=1)


def _join_host_port(host: str, port: int | str):
    """Adapted golang's net.JoinHostPort"""
    template = "%s:%s"
    host_requires_bracketing = ":" in host or "%" in host
    if host_requires_bracketing:
        template = "[%s]:%s"
    return template % (host, port)


class InClusterConfigLoader(object):
    def __init__(
        self,
        token_filename: str,
        cert_filename: str,
        try_refresh_token: bool = True,
        environ: os._Environ | dict[str, str] = os.environ,
    ) -> None:
        self._token_filename = token_filename
        self._cert_filename = cert_filename
        self._environ = environ
        self._try_refresh_token = try_refresh_token

    def load_and_set(self, client_configuration: Configuration | None = None) -> None:
        self._load_config()
        if client_configuration:
            self._set_config(client_configuration)
        else:
            configuration = Configuration()
            self._set_config(configuration)
            Configuration.set_default(configuration)

    def _load_config(self) -> None:
        if (
            SERVICE_HOST_ENV_NAME not in self._environ
            or SERVICE_PORT_ENV_NAME not in self._environ
        ):
            raise ConfigException("Service host/port is not set.")

        if (
            not self._environ[SERVICE_HOST_ENV_NAME]
            or not self._environ[SERVICE_PORT_ENV_NAME]
        ):
            raise ConfigException("Service host/port is set but empty.")

        self.host = "https://" + _join_host_port(
            self._environ[SERVICE_HOST_ENV_NAME], self._environ[SERVICE_PORT_ENV_NAME]
        )

        if not os.path.isfile(self._token_filename):
            raise ConfigException("Service token file does not exist.")

        self._read_token_file()

        if not os.path.isfile(self._cert_filename):
            raise ConfigException("Service certification file does not exists.")

        with open(self._cert_filename) as f:
            if not f.read():
                raise ConfigException("Cert file exists but empty.")

        self.ssl_ca_cert = self._cert_filename

    def _set_config(self, configuration: Configuration) -> None:
        configuration.host = self.host
        configuration.ssl_ca_cert = self.ssl_ca_cert
        if self.token is not None:
            configuration.api_key["BearerToken"] = self.token
        if not self._try_refresh_token:
            return

        def load_token_from_file(configuration, *args):
            if self.token_expires_at <= datetime.datetime.now():
                self._read_token_file()

            # expiration time is stored InClusterConfigLoader,
            # thus some copies of Configuration can be outdated.
            if configuration.api_key["BearerToken"] != self.token:
                configuration.api_key["BearerToken"] = self.token

        configuration.refresh_api_key_hook = load_token_from_file

    def _read_token_file(self):
        with open(self._token_filename) as f:
            content = f.read()
            if not content:
                raise ConfigException("Token file exists but empty.")
            self.token = "Bearer " + content
            self.token_expires_at = datetime.datetime.now() + TOKEN_REFRESH_PERIOD


def load_incluster_config(client_configuration=None, try_refresh_token=True, **kwargs):
    """Use the service account kubernetes gives to pods to connect to kubernetes
    cluster. It's intended for clients that expect to be running inside a pod
    running on kubernetes. It will raise an exception if called from a process
    not running in a kubernetes environment.

    :param client_configuration: The kubernetes.client.Configuration to
    set configs to.
    """
    kwargs.setdefault("token_filename", SERVICE_TOKEN_FILENAME)
    kwargs.setdefault("cert_filename", SERVICE_CERT_FILENAME)
    InClusterConfigLoader(try_refresh_token=try_refresh_token, **kwargs).load_and_set(
        client_configuration
    )



from typing import Any

class V1NodeRuntimeHandlerFeatures:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, recursive_read_only_mounts: bool | None = None, user_namespaces: bool | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def recursive_read_only_mounts(self) -> bool: ...
    @recursive_read_only_mounts.setter
    def recursive_read_only_mounts(self, recursive_read_only_mounts: bool) -> None: ...
    @property
    def user_namespaces(self) -> bool: ...
    @user_namespaces.setter
    def user_namespaces(self, user_namespaces: bool) -> None: ...



from kubernetes_asyncio.client.models import V1NodeAffinity
from kubernetes_asyncio.client.models import V1PodAffinity
from kubernetes_asyncio.client.models import V1PodAntiAffinity
from typing import Any

class V1Affinity:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, node_affinity: V1NodeAffinity | None = None, pod_affinity: V1PodAffinity | None = None, pod_anti_affinity: V1PodAntiAffinity | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def node_affinity(self) -> V1NodeAffinity: ...
    @node_affinity.setter
    def node_affinity(self, node_affinity: V1NodeAffinity) -> None: ...
    @property
    def pod_affinity(self) -> V1PodAffinity: ...
    @pod_affinity.setter
    def pod_affinity(self, pod_affinity: V1PodAffinity) -> None: ...
    @property
    def pod_anti_affinity(self) -> V1PodAntiAffinity: ...
    @pod_anti_affinity.setter
    def pod_anti_affinity(self, pod_anti_affinity: V1PodAntiAffinity) -> None: ...



from typing import Any

class V1ListMeta:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, _continue: str | None = None, remaining_item_count: int | None = None, resource_version: str | None = None, self_link: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def _continue(self) -> str: ...
    @_continue.setter
    def _continue(self, _continue: str) -> None: ...
    @property
    def remaining_item_count(self) -> int: ...
    @remaining_item_count.setter
    def remaining_item_count(self, remaining_item_count: int) -> None: ...
    @property
    def resource_version(self) -> str: ...
    @resource_version.setter
    def resource_version(self, resource_version: str) -> None: ...
    @property
    def self_link(self) -> str: ...
    @self_link.setter
    def self_link(self, self_link: str) -> None: ...



from kubernetes_asyncio.client.models import V1alpha1ServerStorageVersion
from kubernetes_asyncio.client.models import V1alpha1StorageVersionCondition
from typing import Any

class V1alpha1StorageVersionStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, common_encoding_version: str | None = None, conditions: list[V1alpha1StorageVersionCondition] | None = None, storage_versions: list[V1alpha1ServerStorageVersion] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def common_encoding_version(self) -> str: ...
    @common_encoding_version.setter
    def common_encoding_version(self, common_encoding_version: str) -> None: ...
    @property
    def conditions(self) -> list[V1alpha1StorageVersionCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1alpha1StorageVersionCondition]) -> None: ...
    @property
    def storage_versions(self) -> list[V1alpha1ServerStorageVersion]: ...
    @storage_versions.setter
    def storage_versions(self, storage_versions: list[V1alpha1ServerStorageVersion]) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1beta2DeviceClass
from kubernetes_asyncio.client.models import V1beta2DeviceClassList
from kubernetes_asyncio.client.models import V1beta2ResourceClaim
from kubernetes_asyncio.client.models import V1beta2ResourceClaimList
from kubernetes_asyncio.client.models import V1beta2ResourceClaimTemplate
from kubernetes_asyncio.client.models import V1beta2ResourceClaimTemplateList
from kubernetes_asyncio.client.models import V1beta2ResourceSlice
from kubernetes_asyncio.client.models import V1beta2ResourceSliceList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class ResourceV1beta2Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_device_class(self, body: V1beta2DeviceClass, **kwargs: Any) -> Awaitable[V1beta2DeviceClass]: ...
    def create_device_class_with_http_info(self, body: V1beta2DeviceClass, **kwargs: Any) -> Awaitable[tuple[V1beta2DeviceClass, int, CIMultiDictProxy]]: ...
    def create_namespaced_resource_claim(self, namespace: str, body: V1beta2ResourceClaim, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def create_namespaced_resource_claim_template(self, namespace: str, body: V1beta2ResourceClaimTemplate, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplate]: ...
    def create_namespaced_resource_claim_template_with_http_info(self, namespace: str, body: V1beta2ResourceClaimTemplate, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplate, int, CIMultiDictProxy]]: ...
    def create_namespaced_resource_claim_with_http_info(self, namespace: str, body: V1beta2ResourceClaim, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def create_resource_slice(self, body: V1beta2ResourceSlice, **kwargs: Any) -> Awaitable[V1beta2ResourceSlice]: ...
    def create_resource_slice_with_http_info(self, body: V1beta2ResourceSlice, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceSlice, int, CIMultiDictProxy]]: ...
    def delete_collection_device_class(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_device_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_resource_claim(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_resource_claim_template(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_resource_claim_template_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_resource_claim_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_resource_slice(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_resource_slice_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_device_class(self, name: str, **kwargs: Any) -> Awaitable[V1beta2DeviceClass]: ...
    def delete_device_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta2DeviceClass, int, CIMultiDictProxy]]: ...
    def delete_namespaced_resource_claim(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def delete_namespaced_resource_claim_template(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplate]: ...
    def delete_namespaced_resource_claim_template_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplate, int, CIMultiDictProxy]]: ...
    def delete_namespaced_resource_claim_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def delete_resource_slice(self, name: str, **kwargs: Any) -> Awaitable[V1beta2ResourceSlice]: ...
    def delete_resource_slice_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceSlice, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_device_class(self, **kwargs: Any) -> Awaitable[V1beta2DeviceClassList]: ...
    def list_device_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta2DeviceClassList, int, CIMultiDictProxy]]: ...
    def list_namespaced_resource_claim(self, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimList]: ...
    def list_namespaced_resource_claim_template(self, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplateList]: ...
    def list_namespaced_resource_claim_template_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplateList, int, CIMultiDictProxy]]: ...
    def list_namespaced_resource_claim_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimList, int, CIMultiDictProxy]]: ...
    def list_resource_claim_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimList]: ...
    def list_resource_claim_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimList, int, CIMultiDictProxy]]: ...
    def list_resource_claim_template_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplateList]: ...
    def list_resource_claim_template_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplateList, int, CIMultiDictProxy]]: ...
    def list_resource_slice(self, **kwargs: Any) -> Awaitable[V1beta2ResourceSliceList]: ...
    def list_resource_slice_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceSliceList, int, CIMultiDictProxy]]: ...
    def patch_device_class(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta2DeviceClass]: ...
    def patch_device_class_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta2DeviceClass, int, CIMultiDictProxy]]: ...
    def patch_namespaced_resource_claim(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def patch_namespaced_resource_claim_status(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def patch_namespaced_resource_claim_status_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def patch_namespaced_resource_claim_template(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplate]: ...
    def patch_namespaced_resource_claim_template_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplate, int, CIMultiDictProxy]]: ...
    def patch_namespaced_resource_claim_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def patch_resource_slice(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta2ResourceSlice]: ...
    def patch_resource_slice_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceSlice, int, CIMultiDictProxy]]: ...
    def read_device_class(self, name: str, **kwargs: Any) -> Awaitable[V1beta2DeviceClass]: ...
    def read_device_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta2DeviceClass, int, CIMultiDictProxy]]: ...
    def read_namespaced_resource_claim(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def read_namespaced_resource_claim_status(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def read_namespaced_resource_claim_status_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def read_namespaced_resource_claim_template(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplate]: ...
    def read_namespaced_resource_claim_template_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplate, int, CIMultiDictProxy]]: ...
    def read_namespaced_resource_claim_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def read_resource_slice(self, name: str, **kwargs: Any) -> Awaitable[V1beta2ResourceSlice]: ...
    def read_resource_slice_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceSlice, int, CIMultiDictProxy]]: ...
    def replace_device_class(self, name: str, body: V1beta2DeviceClass, **kwargs: Any) -> Awaitable[V1beta2DeviceClass]: ...
    def replace_device_class_with_http_info(self, name: str, body: V1beta2DeviceClass, **kwargs: Any) -> Awaitable[tuple[V1beta2DeviceClass, int, CIMultiDictProxy]]: ...
    def replace_namespaced_resource_claim(self, name: str, namespace: str, body: V1beta2ResourceClaim, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def replace_namespaced_resource_claim_status(self, name: str, namespace: str, body: V1beta2ResourceClaim, **kwargs: Any) -> Awaitable[V1beta2ResourceClaim]: ...
    def replace_namespaced_resource_claim_status_with_http_info(self, name: str, namespace: str, body: V1beta2ResourceClaim, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def replace_namespaced_resource_claim_template(self, name: str, namespace: str, body: V1beta2ResourceClaimTemplate, **kwargs: Any) -> Awaitable[V1beta2ResourceClaimTemplate]: ...
    def replace_namespaced_resource_claim_template_with_http_info(self, name: str, namespace: str, body: V1beta2ResourceClaimTemplate, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaimTemplate, int, CIMultiDictProxy]]: ...
    def replace_namespaced_resource_claim_with_http_info(self, name: str, namespace: str, body: V1beta2ResourceClaim, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceClaim, int, CIMultiDictProxy]]: ...
    def replace_resource_slice(self, name: str, body: V1beta2ResourceSlice, **kwargs: Any) -> Awaitable[V1beta2ResourceSlice]: ...
    def replace_resource_slice_with_http_info(self, name: str, body: V1beta2ResourceSlice, **kwargs: Any) -> Awaitable[tuple[V1beta2ResourceSlice, int, CIMultiDictProxy]]: ...



from typing import Any

class V1CustomResourceColumnDefinition:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, description: str | None = None, format: str | None = None, json_path: str | None = None, name: str | None = None, priority: int | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, description: str) -> None: ...
    @property
    def format(self) -> str: ...
    @format.setter
    def format(self, format: str) -> None: ...
    @property
    def json_path(self) -> str: ...
    @json_path.setter
    def json_path(self, json_path: str) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def priority(self) -> int: ...
    @priority.setter
    def priority(self, priority: int) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V1ExemptPriorityLevelConfiguration
from kubernetes_asyncio.client.models import V1LimitedPriorityLevelConfiguration
from typing import Any

class V1PriorityLevelConfigurationSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, exempt: V1ExemptPriorityLevelConfiguration | None = None, limited: V1LimitedPriorityLevelConfiguration | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def exempt(self) -> V1ExemptPriorityLevelConfiguration: ...
    @exempt.setter
    def exempt(self, exempt: V1ExemptPriorityLevelConfiguration) -> None: ...
    @property
    def limited(self) -> V1LimitedPriorityLevelConfiguration: ...
    @limited.setter
    def limited(self, limited: V1LimitedPriorityLevelConfiguration) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V1ContainerPort
from kubernetes_asyncio.client.models import V1ContainerResizePolicy
from kubernetes_asyncio.client.models import V1ContainerRestartRule
from kubernetes_asyncio.client.models import V1EnvFromSource
from kubernetes_asyncio.client.models import V1EnvVar
from kubernetes_asyncio.client.models import V1Lifecycle
from kubernetes_asyncio.client.models import V1Probe
from kubernetes_asyncio.client.models import V1ResourceRequirements
from kubernetes_asyncio.client.models import V1SecurityContext
from kubernetes_asyncio.client.models import V1VolumeDevice
from kubernetes_asyncio.client.models import V1VolumeMount
from typing import Any

class V1Container:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, args: list[str] | None = None, command: list[str] | None = None, env: list[V1EnvVar] | None = None, env_from: list[V1EnvFromSource] | None = None, image: str | None = None, image_pull_policy: str | None = None, lifecycle: V1Lifecycle | None = None, liveness_probe: V1Probe | None = None, name: str | None = None, ports: list[V1ContainerPort] | None = None, readiness_probe: V1Probe | None = None, resize_policy: list[V1ContainerResizePolicy] | None = None, resources: V1ResourceRequirements | None = None, restart_policy: str | None = None, restart_policy_rules: list[V1ContainerRestartRule] | None = None, security_context: V1SecurityContext | None = None, startup_probe: V1Probe | None = None, stdin: bool | None = None, stdin_once: bool | None = None, termination_message_path: str | None = None, termination_message_policy: str | None = None, tty: bool | None = None, volume_devices: list[V1VolumeDevice] | None = None, volume_mounts: list[V1VolumeMount] | None = None, working_dir: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def args(self) -> list[str]: ...
    @args.setter
    def args(self, args: list[str]) -> None: ...
    @property
    def command(self) -> list[str]: ...
    @command.setter
    def command(self, command: list[str]) -> None: ...
    @property
    def env(self) -> list[V1EnvVar]: ...
    @env.setter
    def env(self, env: list[V1EnvVar]) -> None: ...
    @property
    def env_from(self) -> list[V1EnvFromSource]: ...
    @env_from.setter
    def env_from(self, env_from: list[V1EnvFromSource]) -> None: ...
    @property
    def image(self) -> str: ...
    @image.setter
    def image(self, image: str) -> None: ...
    @property
    def image_pull_policy(self) -> str: ...
    @image_pull_policy.setter
    def image_pull_policy(self, image_pull_policy: str) -> None: ...
    @property
    def lifecycle(self) -> V1Lifecycle: ...
    @lifecycle.setter
    def lifecycle(self, lifecycle: V1Lifecycle) -> None: ...
    @property
    def liveness_probe(self) -> V1Probe: ...
    @liveness_probe.setter
    def liveness_probe(self, liveness_probe: V1Probe) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def ports(self) -> list[V1ContainerPort]: ...
    @ports.setter
    def ports(self, ports: list[V1ContainerPort]) -> None: ...
    @property
    def readiness_probe(self) -> V1Probe: ...
    @readiness_probe.setter
    def readiness_probe(self, readiness_probe: V1Probe) -> None: ...
    @property
    def resize_policy(self) -> list[V1ContainerResizePolicy]: ...
    @resize_policy.setter
    def resize_policy(self, resize_policy: list[V1ContainerResizePolicy]) -> None: ...
    @property
    def resources(self) -> V1ResourceRequirements: ...
    @resources.setter
    def resources(self, resources: V1ResourceRequirements) -> None: ...
    @property
    def restart_policy(self) -> str: ...
    @restart_policy.setter
    def restart_policy(self, restart_policy: str) -> None: ...
    @property
    def restart_policy_rules(self) -> list[V1ContainerRestartRule]: ...
    @restart_policy_rules.setter
    def restart_policy_rules(self, restart_policy_rules: list[V1ContainerRestartRule]) -> None: ...
    @property
    def security_context(self) -> V1SecurityContext: ...
    @security_context.setter
    def security_context(self, security_context: V1SecurityContext) -> None: ...
    @property
    def startup_probe(self) -> V1Probe: ...
    @startup_probe.setter
    def startup_probe(self, startup_probe: V1Probe) -> None: ...
    @property
    def stdin(self) -> bool: ...
    @stdin.setter
    def stdin(self, stdin: bool) -> None: ...
    @property
    def stdin_once(self) -> bool: ...
    @stdin_once.setter
    def stdin_once(self, stdin_once: bool) -> None: ...
    @property
    def termination_message_path(self) -> str: ...
    @termination_message_path.setter
    def termination_message_path(self, termination_message_path: str) -> None: ...
    @property
    def termination_message_policy(self) -> str: ...
    @termination_message_policy.setter
    def termination_message_policy(self, termination_message_policy: str) -> None: ...
    @property
    def tty(self) -> bool: ...
    @tty.setter
    def tty(self, tty: bool) -> None: ...
    @property
    def volume_devices(self) -> list[V1VolumeDevice]: ...
    @volume_devices.setter
    def volume_devices(self, volume_devices: list[V1VolumeDevice]) -> None: ...
    @property
    def volume_mounts(self) -> list[V1VolumeMount]: ...
    @volume_mounts.setter
    def volume_mounts(self, volume_mounts: list[V1VolumeMount]) -> None: ...
    @property
    def working_dir(self) -> str: ...
    @working_dir.setter
    def working_dir(self, working_dir: str) -> None: ...



from kubernetes_asyncio.client.models import V1VolumeAttachmentSource
from typing import Any

class V1VolumeAttachmentSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, attacher: str | None = None, node_name: str | None = None, source: V1VolumeAttachmentSource | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def attacher(self) -> str: ...
    @attacher.setter
    def attacher(self, attacher: str) -> None: ...
    @property
    def node_name(self) -> str: ...
    @node_name.setter
    def node_name(self, node_name: str) -> None: ...
    @property
    def source(self) -> V1VolumeAttachmentSource: ...
    @source.setter
    def source(self, source: V1VolumeAttachmentSource) -> None: ...



# Copyright 2016 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import json
import pydoc
from functools import partial
from types import SimpleNamespace

from kubernetes_asyncio.client import ApiClient
from kubernetes_asyncio.client.exceptions import ApiException
from kubernetes_asyncio.config import Any

PYDOC_RETURN_LABEL = ":rtype:"
PYDOC_FOLLOW_PARAM = ":param follow:"

# Removing this suffix from return type name should give us event's object
# type. e.g., if list_namespaces() returns "NamespaceList" type,
# then list_namespaces(watch=true) returns a stream of events with objects
# of type "Namespace". In case this assumption is not true, user should
# provide return_type to Watch class's __init__.
TYPE_LIST_SUFFIX = "List"


def _find_return_type(func: object) -> str:
    for line in pydoc.getdoc(func).splitlines():
        if line.startswith(PYDOC_RETURN_LABEL):
            return line[len(PYDOC_RETURN_LABEL):].strip()
    return ""


class Stream:
    def __init__(self, func, *args, **kwargs):
        pass


class Watch:
    def __init__(self, return_type: str | None = None) -> None:
        self._raw_return_type = return_type
        self._stop = False
        self._api_client = ApiClient()
        self.resource_version = None
        self.resp = None

    def stop(self) -> None:
        self._stop = True

    def get_return_type(self, func: object) -> str:
        if self._raw_return_type:
            return self._raw_return_type
        return_type = _find_return_type(func)

        if return_type.endswith(TYPE_LIST_SUFFIX):
            return return_type[: -len(TYPE_LIST_SUFFIX)]
        return return_type

    def get_watch_argument_name(self, func: object) -> str:
        if PYDOC_FOLLOW_PARAM in pydoc.getdoc(func):
            return "follow"
        else:
            return "watch"

    def unmarshal_event(self, data: str | bytes, response_type: str | None) -> Any:
        """Return the K8s response `data` in JSON format."""
        try:
            js = json.loads(data)
        except ValueError:
            return data

        if "object" not in js or "type" not in js:
            # raise error with code if set
            if "code" in js:
                reason = "{}: {}".format(js.get("reason"), js.get("message"))
                raise ApiException(status=js["code"], reason=reason)

            raise Exception(
                (
                    "Malformed JSON response, the 'object' and/or "
                    "'type' field is missing. JSON: {}"
                ).format(js)
            )
        # Make a copy of the original object and save it under the
        # `raw_object` key because we will replace the data under `object` with
        # a Python native type shortly.
        js["raw_object"] = js["object"]

        # Something went wrong. A typical example would be that the user
        # supplied a resource version that was too old. In that case K8s would
        # not send a conventional ADDED/DELETED/... event but an error. Turn
        # this error into a Python exception to save the user the hassle.
        if js["type"].lower() == "error":
            obj = js["raw_object"]
            reason = "{}: {}".format(obj["reason"], obj["message"])
            raise ApiException(status=obj["code"], reason=reason)

        if js["type"].lower() != "bookmark":
            # If possible, compile the JSON response into a Python native response
            # type, eg `V1Namespace` or `V1Pod`,`ExtensionsV1beta1Deployment`, ...
            if response_type:
                js["object"] = self._api_client.deserialize(
                    response=SimpleNamespace(data=json.dumps(js["raw_object"])),  # type: ignore
                    response_type=response_type,
                )

            # decode and save resource_version to continue watching
            if hasattr(js["object"], "metadata"):
                self.resource_version = js["object"].metadata.resource_version

            # For custom objects that we don't have model defined, json
            # deserialization results in dictionary
            elif (
                isinstance(js["object"], dict)
                and "metadata" in js["object"]
                and "resourceVersion" in js["object"]["metadata"]
            ):
                self.resource_version = js["object"]["metadata"]["resourceVersion"]

        elif js["type"].lower() == "bookmark":
            if (
                isinstance(js["raw_object"], dict)
                and "metadata" in js["raw_object"]
                and "resourceVersion" in js["raw_object"]["metadata"]
            ):
                self.resource_version = js["raw_object"]["metadata"]["resourceVersion"]
            else:
                raise Exception(
                    (
                        "Malformed JSON response for bookmark event, "
                        "'metadata' or 'resourceVersion' field is missing. "
                        "JSON: {}"
                    ).format(js)
                )

        return js

    def __aiter__(self) -> "Watch":
        return self

    async def __anext__(self) -> Any:
        try:
            return await self.next()
        except:  # noqa: E722
            await self.close()
            raise

    def _reconnect(self) -> None:
        if self.resp:
            self.resp.close()
            self.resp = None
        if self.resource_version:
            self.func.keywords["resource_version"] = self.resource_version

    async def next(self) -> Any:
        watch_forever = "timeout_seconds" not in self.func.keywords
        retry_410 = watch_forever

        while 1:
            # Set the response object to the user supplied function (eg
            # `list_namespaced_pods`) if this is the first iteration.
            if self.resp is None:
                self.resp = await self.func()

            # Abort at the current iteration if the user has called `stop` on this
            # stream instance.
            if self._stop:
                raise StopAsyncIteration

            # Fetch the next K8s response.
            try:
                if self.resp is None:
                    continue
                line = await self.resp.content.readline()
            except asyncio.TimeoutError:
                # This exception can be raised by aiohttp (client timeout)
                # but we don't retry if server side timeout is applied.
                if watch_forever:
                    self._reconnect()
                    continue
                else:
                    raise

            line = line.decode("utf8")

            # Special case for faster log streaming
            if self.return_type == "str":
                if line == "":
                    # end of log
                    raise StopAsyncIteration
                return line

            # Stop the iterator if K8s sends an empty response. This happens when
            # eg the supplied timeout has expired.
            if line == "":
                if watch_forever:
                    self._reconnect()
                    continue
                raise StopAsyncIteration

            # retry 410 error only once
            try:
                event = self.unmarshal_event(line, self.return_type)
            except ApiException as ex:
                if ex.status == 410 and retry_410:
                    retry_410 = False  # retry only once
                    self._reconnect()
                    continue
                raise
            retry_410 = watch_forever
            return event

    def stream(self, func, *args, **kwargs) -> "Watch":
        """Watch an API resource and stream the result back via a generator.

        :param func: The API function pointer. Any parameter to the function
                     can be passed after this parameter.

        :return: Event object with these keys:
                   'type': The type of event such as "ADDED", "DELETED", etc.
                   'raw_object': a dict representing the watched object.
                   'object': A model representation of raw_object. The name of
                             model will be determined based on
                             the func's doc string. If it cannot be determined,
                             'object' value will be the same as 'raw_object'.

        Example:
            v1 = kubernetes_asyncio.client.CoreV1Api()
            watch = kubernetes_asyncio.watch.Watch()
            async for e in watch.stream(v1.list_namespace, timeout_seconds=10):
                type = e['type']
                object = e['object']  # object is one of type return_type
                raw_object = e['raw_object']  # raw_object is a dict
                ...
                if should_stop:
                    watch.stop()
        """
        self._stop = False
        self.return_type = self.get_return_type(func)
        kwargs[self.get_watch_argument_name(func)] = True
        kwargs["_preload_content"] = False
        if "resource_version" in kwargs:
            self.resource_version = kwargs["resource_version"]

        self.func = partial(func, *args, **kwargs)

        return self

    async def __aenter__(self) -> "Watch":
        return self

    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
        await self.close()

    async def close(self) -> None:
        await self._api_client.close()
        if self.resp is not None:
            self.resp.release()
            self.resp = None



from typing import Any

class V1SELinuxOptions:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, level: str | None = None, role: str | None = None, type: str | None = None, user: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def level(self) -> str: ...
    @level.setter
    def level(self, level: str) -> None: ...
    @property
    def role(self) -> str: ...
    @role.setter
    def role(self, role: str) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...
    @property
    def user(self) -> str: ...
    @user.setter
    def user(self, user: str) -> None: ...



from typing import Any

class V1ExemptPriorityLevelConfiguration:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, lendable_percent: int | None = None, nominal_concurrency_shares: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def lendable_percent(self) -> int: ...
    @lendable_percent.setter
    def lendable_percent(self, lendable_percent: int) -> None: ...
    @property
    def nominal_concurrency_shares(self) -> int: ...
    @nominal_concurrency_shares.setter
    def nominal_concurrency_shares(self, nominal_concurrency_shares: int) -> None: ...



from datetime import datetime
from kubernetes_asyncio.client.models import EventsV1EventSeries
from kubernetes_asyncio.client.models import V1EventSource
from kubernetes_asyncio.client.models import V1ObjectMeta
from kubernetes_asyncio.client.models import V1ObjectReference
from typing import Any

class EventsV1Event:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, action: str | None = None, api_version: str | None = None, deprecated_count: int | None = None, deprecated_first_timestamp: datetime | None = None, deprecated_last_timestamp: datetime | None = None, deprecated_source: V1EventSource | None = None, event_time: datetime | None = None, kind: str | None = None, metadata: V1ObjectMeta | None = None, note: str | None = None, reason: str | None = None, regarding: V1ObjectReference | None = None, related: V1ObjectReference | None = None, reporting_controller: str | None = None, reporting_instance: str | None = None, series: EventsV1EventSeries | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def action(self) -> str: ...
    @action.setter
    def action(self, action: str) -> None: ...
    @property
    def api_version(self) -> str: ...
    @api_version.setter
    def api_version(self, api_version: str) -> None: ...
    @property
    def deprecated_count(self) -> int: ...
    @deprecated_count.setter
    def deprecated_count(self, deprecated_count: int) -> None: ...
    @property
    def deprecated_first_timestamp(self) -> datetime: ...
    @deprecated_first_timestamp.setter
    def deprecated_first_timestamp(self, deprecated_first_timestamp: datetime) -> None: ...
    @property
    def deprecated_last_timestamp(self) -> datetime: ...
    @deprecated_last_timestamp.setter
    def deprecated_last_timestamp(self, deprecated_last_timestamp: datetime) -> None: ...
    @property
    def deprecated_source(self) -> V1EventSource: ...
    @deprecated_source.setter
    def deprecated_source(self, deprecated_source: V1EventSource) -> None: ...
    @property
    def event_time(self) -> datetime: ...
    @event_time.setter
    def event_time(self, event_time: datetime) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def metadata(self) -> V1ObjectMeta: ...
    @metadata.setter
    def metadata(self, metadata: V1ObjectMeta) -> None: ...
    @property
    def note(self) -> str: ...
    @note.setter
    def note(self, note: str) -> None: ...
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, reason: str) -> None: ...
    @property
    def regarding(self) -> V1ObjectReference: ...
    @regarding.setter
    def regarding(self, regarding: V1ObjectReference) -> None: ...
    @property
    def related(self) -> V1ObjectReference: ...
    @related.setter
    def related(self, related: V1ObjectReference) -> None: ...
    @property
    def reporting_controller(self) -> str: ...
    @reporting_controller.setter
    def reporting_controller(self, reporting_controller: str) -> None: ...
    @property
    def reporting_instance(self) -> str: ...
    @reporting_instance.setter
    def reporting_instance(self, reporting_instance: str) -> None: ...
    @property
    def series(self) -> EventsV1EventSeries: ...
    @series.setter
    def series(self, series: EventsV1EventSeries) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V1beta1DeviceAllocationConfiguration
from kubernetes_asyncio.client.models import V1beta1DeviceRequestAllocationResult
from typing import Any

class V1beta1DeviceAllocationResult:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, config: list[V1beta1DeviceAllocationConfiguration] | None = None, results: list[V1beta1DeviceRequestAllocationResult] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def config(self) -> list[V1beta1DeviceAllocationConfiguration]: ...
    @config.setter
    def config(self, config: list[V1beta1DeviceAllocationConfiguration]) -> None: ...
    @property
    def results(self) -> list[V1beta1DeviceRequestAllocationResult]: ...
    @results.setter
    def results(self, results: list[V1beta1DeviceRequestAllocationResult]) -> None: ...



from kubernetes_asyncio.client.models import V1ExpressionWarning
from typing import Any

class V1TypeChecking:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, expression_warnings: list[V1ExpressionWarning] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def expression_warnings(self) -> list[V1ExpressionWarning]: ...
    @expression_warnings.setter
    def expression_warnings(self, expression_warnings: list[V1ExpressionWarning]) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1ControllerRevision
from kubernetes_asyncio.client.models import V1ControllerRevisionList
from kubernetes_asyncio.client.models import V1DaemonSet
from kubernetes_asyncio.client.models import V1DaemonSetList
from kubernetes_asyncio.client.models import V1Deployment
from kubernetes_asyncio.client.models import V1DeploymentList
from kubernetes_asyncio.client.models import V1ReplicaSet
from kubernetes_asyncio.client.models import V1ReplicaSetList
from kubernetes_asyncio.client.models import V1Scale
from kubernetes_asyncio.client.models import V1StatefulSet
from kubernetes_asyncio.client.models import V1StatefulSetList
from kubernetes_asyncio.client.models import V1Status
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class AppsV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_namespaced_controller_revision(self, namespace: str, body: V1ControllerRevision, **kwargs: Any) -> Awaitable[V1ControllerRevision]: ...
    def create_namespaced_controller_revision_with_http_info(self, namespace: str, body: V1ControllerRevision, **kwargs: Any) -> Awaitable[tuple[V1ControllerRevision, int, CIMultiDictProxy]]: ...
    def create_namespaced_daemon_set(self, namespace: str, body: V1DaemonSet, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def create_namespaced_daemon_set_with_http_info(self, namespace: str, body: V1DaemonSet, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def create_namespaced_deployment(self, namespace: str, body: V1Deployment, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def create_namespaced_deployment_with_http_info(self, namespace: str, body: V1Deployment, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def create_namespaced_replica_set(self, namespace: str, body: V1ReplicaSet, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def create_namespaced_replica_set_with_http_info(self, namespace: str, body: V1ReplicaSet, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def create_namespaced_stateful_set(self, namespace: str, body: V1StatefulSet, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def create_namespaced_stateful_set_with_http_info(self, namespace: str, body: V1StatefulSet, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_controller_revision(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_controller_revision_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_daemon_set(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_daemon_set_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_deployment(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_deployment_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_replica_set(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_replica_set_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_stateful_set(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_stateful_set_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_controller_revision(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_controller_revision_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_daemon_set(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_daemon_set_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_deployment(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_deployment_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_replica_set(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_replica_set_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_stateful_set(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_stateful_set_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_controller_revision_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1ControllerRevisionList]: ...
    def list_controller_revision_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ControllerRevisionList, int, CIMultiDictProxy]]: ...
    def list_daemon_set_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1DaemonSetList]: ...
    def list_daemon_set_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1DaemonSetList, int, CIMultiDictProxy]]: ...
    def list_deployment_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1DeploymentList]: ...
    def list_deployment_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1DeploymentList, int, CIMultiDictProxy]]: ...
    def list_namespaced_controller_revision(self, namespace: str, **kwargs: Any) -> Awaitable[V1ControllerRevisionList]: ...
    def list_namespaced_controller_revision_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1ControllerRevisionList, int, CIMultiDictProxy]]: ...
    def list_namespaced_daemon_set(self, namespace: str, **kwargs: Any) -> Awaitable[V1DaemonSetList]: ...
    def list_namespaced_daemon_set_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1DaemonSetList, int, CIMultiDictProxy]]: ...
    def list_namespaced_deployment(self, namespace: str, **kwargs: Any) -> Awaitable[V1DeploymentList]: ...
    def list_namespaced_deployment_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1DeploymentList, int, CIMultiDictProxy]]: ...
    def list_namespaced_replica_set(self, namespace: str, **kwargs: Any) -> Awaitable[V1ReplicaSetList]: ...
    def list_namespaced_replica_set_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSetList, int, CIMultiDictProxy]]: ...
    def list_namespaced_stateful_set(self, namespace: str, **kwargs: Any) -> Awaitable[V1StatefulSetList]: ...
    def list_namespaced_stateful_set_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1StatefulSetList, int, CIMultiDictProxy]]: ...
    def list_replica_set_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1ReplicaSetList]: ...
    def list_replica_set_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSetList, int, CIMultiDictProxy]]: ...
    def list_stateful_set_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1StatefulSetList]: ...
    def list_stateful_set_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1StatefulSetList, int, CIMultiDictProxy]]: ...
    def patch_namespaced_controller_revision(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1ControllerRevision]: ...
    def patch_namespaced_controller_revision_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ControllerRevision, int, CIMultiDictProxy]]: ...
    def patch_namespaced_daemon_set(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def patch_namespaced_daemon_set_status(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def patch_namespaced_daemon_set_status_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def patch_namespaced_daemon_set_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def patch_namespaced_deployment(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def patch_namespaced_deployment_scale(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def patch_namespaced_deployment_scale_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def patch_namespaced_deployment_status(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def patch_namespaced_deployment_status_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def patch_namespaced_deployment_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def patch_namespaced_replica_set(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def patch_namespaced_replica_set_scale(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def patch_namespaced_replica_set_scale_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def patch_namespaced_replica_set_status(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def patch_namespaced_replica_set_status_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def patch_namespaced_replica_set_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def patch_namespaced_stateful_set(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def patch_namespaced_stateful_set_scale(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def patch_namespaced_stateful_set_scale_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def patch_namespaced_stateful_set_status(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def patch_namespaced_stateful_set_status_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...
    def patch_namespaced_stateful_set_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...
    def read_namespaced_controller_revision(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1ControllerRevision]: ...
    def read_namespaced_controller_revision_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1ControllerRevision, int, CIMultiDictProxy]]: ...
    def read_namespaced_daemon_set(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def read_namespaced_daemon_set_status(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def read_namespaced_daemon_set_status_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def read_namespaced_daemon_set_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def read_namespaced_deployment(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def read_namespaced_deployment_scale(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def read_namespaced_deployment_scale_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def read_namespaced_deployment_status(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def read_namespaced_deployment_status_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def read_namespaced_deployment_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def read_namespaced_replica_set(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def read_namespaced_replica_set_scale(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def read_namespaced_replica_set_scale_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def read_namespaced_replica_set_status(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def read_namespaced_replica_set_status_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def read_namespaced_replica_set_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def read_namespaced_stateful_set(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def read_namespaced_stateful_set_scale(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def read_namespaced_stateful_set_scale_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def read_namespaced_stateful_set_status(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def read_namespaced_stateful_set_status_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...
    def read_namespaced_stateful_set_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...
    def replace_namespaced_controller_revision(self, name: str, namespace: str, body: V1ControllerRevision, **kwargs: Any) -> Awaitable[V1ControllerRevision]: ...
    def replace_namespaced_controller_revision_with_http_info(self, name: str, namespace: str, body: V1ControllerRevision, **kwargs: Any) -> Awaitable[tuple[V1ControllerRevision, int, CIMultiDictProxy]]: ...
    def replace_namespaced_daemon_set(self, name: str, namespace: str, body: V1DaemonSet, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def replace_namespaced_daemon_set_status(self, name: str, namespace: str, body: V1DaemonSet, **kwargs: Any) -> Awaitable[V1DaemonSet]: ...
    def replace_namespaced_daemon_set_status_with_http_info(self, name: str, namespace: str, body: V1DaemonSet, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def replace_namespaced_daemon_set_with_http_info(self, name: str, namespace: str, body: V1DaemonSet, **kwargs: Any) -> Awaitable[tuple[V1DaemonSet, int, CIMultiDictProxy]]: ...
    def replace_namespaced_deployment(self, name: str, namespace: str, body: V1Deployment, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def replace_namespaced_deployment_scale(self, name: str, namespace: str, body: V1Scale, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def replace_namespaced_deployment_scale_with_http_info(self, name: str, namespace: str, body: V1Scale, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def replace_namespaced_deployment_status(self, name: str, namespace: str, body: V1Deployment, **kwargs: Any) -> Awaitable[V1Deployment]: ...
    def replace_namespaced_deployment_status_with_http_info(self, name: str, namespace: str, body: V1Deployment, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def replace_namespaced_deployment_with_http_info(self, name: str, namespace: str, body: V1Deployment, **kwargs: Any) -> Awaitable[tuple[V1Deployment, int, CIMultiDictProxy]]: ...
    def replace_namespaced_replica_set(self, name: str, namespace: str, body: V1ReplicaSet, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def replace_namespaced_replica_set_scale(self, name: str, namespace: str, body: V1Scale, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def replace_namespaced_replica_set_scale_with_http_info(self, name: str, namespace: str, body: V1Scale, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def replace_namespaced_replica_set_status(self, name: str, namespace: str, body: V1ReplicaSet, **kwargs: Any) -> Awaitable[V1ReplicaSet]: ...
    def replace_namespaced_replica_set_status_with_http_info(self, name: str, namespace: str, body: V1ReplicaSet, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def replace_namespaced_replica_set_with_http_info(self, name: str, namespace: str, body: V1ReplicaSet, **kwargs: Any) -> Awaitable[tuple[V1ReplicaSet, int, CIMultiDictProxy]]: ...
    def replace_namespaced_stateful_set(self, name: str, namespace: str, body: V1StatefulSet, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def replace_namespaced_stateful_set_scale(self, name: str, namespace: str, body: V1Scale, **kwargs: Any) -> Awaitable[V1Scale]: ...
    def replace_namespaced_stateful_set_scale_with_http_info(self, name: str, namespace: str, body: V1Scale, **kwargs: Any) -> Awaitable[tuple[V1Scale, int, CIMultiDictProxy]]: ...
    def replace_namespaced_stateful_set_status(self, name: str, namespace: str, body: V1StatefulSet, **kwargs: Any) -> Awaitable[V1StatefulSet]: ...
    def replace_namespaced_stateful_set_status_with_http_info(self, name: str, namespace: str, body: V1StatefulSet, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...
    def replace_namespaced_stateful_set_with_http_info(self, name: str, namespace: str, body: V1StatefulSet, **kwargs: Any) -> Awaitable[tuple[V1StatefulSet, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1JSONSchemaProps
from typing import Any

class V1CustomResourceValidation:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, open_apiv3_schema: V1JSONSchemaProps | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def open_apiv3_schema(self) -> V1JSONSchemaProps: ...
    @open_apiv3_schema.setter
    def open_apiv3_schema(self, open_apiv3_schema: V1JSONSchemaProps) -> None: ...



from kubernetes_asyncio.client.models import ApiregistrationV1ServiceReference
from typing import Any

class V1APIServiceSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, ca_bundle: str | None = None, group: str | None = None, group_priority_minimum: int | None = None, insecure_skip_tls_verify: bool | None = None, service: ApiregistrationV1ServiceReference | None = None, version: str | None = None, version_priority: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def ca_bundle(self) -> str: ...
    @ca_bundle.setter
    def ca_bundle(self, ca_bundle: str) -> None: ...
    @property
    def group(self) -> str: ...
    @group.setter
    def group(self, group: str) -> None: ...
    @property
    def group_priority_minimum(self) -> int: ...
    @group_priority_minimum.setter
    def group_priority_minimum(self, group_priority_minimum: int) -> None: ...
    @property
    def insecure_skip_tls_verify(self) -> bool: ...
    @insecure_skip_tls_verify.setter
    def insecure_skip_tls_verify(self, insecure_skip_tls_verify: bool) -> None: ...
    @property
    def service(self) -> ApiregistrationV1ServiceReference: ...
    @service.setter
    def service(self, service: ApiregistrationV1ServiceReference) -> None: ...
    @property
    def version(self) -> str: ...
    @version.setter
    def version(self, version: str) -> None: ...
    @property
    def version_priority(self) -> int: ...
    @version_priority.setter
    def version_priority(self, version_priority: int) -> None: ...



from typing import Any

class V1alpha1ServerStorageVersion:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, api_server_id: str | None = None, decodable_versions: list[str] | None = None, encoding_version: str | None = None, served_versions: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def api_server_id(self) -> str: ...
    @api_server_id.setter
    def api_server_id(self, api_server_id: str) -> None: ...
    @property
    def decodable_versions(self) -> list[str]: ...
    @decodable_versions.setter
    def decodable_versions(self, decodable_versions: list[str]) -> None: ...
    @property
    def encoding_version(self) -> str: ...
    @encoding_version.setter
    def encoding_version(self, encoding_version: str) -> None: ...
    @property
    def served_versions(self) -> list[str]: ...
    @served_versions.setter
    def served_versions(self, served_versions: list[str]) -> None: ...



from kubernetes_asyncio.client.models import V1DeviceClaim
from typing import Any

class V1ResourceClaimSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, devices: V1DeviceClaim | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def devices(self) -> V1DeviceClaim: ...
    @devices.setter
    def devices(self, devices: V1DeviceClaim) -> None: ...



from typing import Any

class V1GlusterfsPersistentVolumeSource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, endpoints: str | None = None, endpoints_namespace: str | None = None, path: str | None = None, read_only: bool | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def endpoints(self) -> str: ...
    @endpoints.setter
    def endpoints(self, endpoints: str) -> None: ...
    @property
    def endpoints_namespace(self) -> str: ...
    @endpoints_namespace.setter
    def endpoints_namespace(self, endpoints_namespace: str) -> None: ...
    @property
    def path(self) -> str: ...
    @path.setter
    def path(self, path: str) -> None: ...
    @property
    def read_only(self) -> bool: ...
    @read_only.setter
    def read_only(self, read_only: bool) -> None: ...



from datetime import datetime
from kubernetes_asyncio.client.models import V1NodeSelector
from kubernetes_asyncio.client.models import V1beta2DeviceAllocationResult
from typing import Any

class V1beta2AllocationResult:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, allocation_timestamp: datetime | None = None, devices: V1beta2DeviceAllocationResult | None = None, node_selector: V1NodeSelector | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def allocation_timestamp(self) -> datetime: ...
    @allocation_timestamp.setter
    def allocation_timestamp(self, allocation_timestamp: datetime) -> None: ...
    @property
    def devices(self) -> V1beta2DeviceAllocationResult: ...
    @devices.setter
    def devices(self, devices: V1beta2DeviceAllocationResult) -> None: ...
    @property
    def node_selector(self) -> V1NodeSelector: ...
    @node_selector.setter
    def node_selector(self, node_selector: V1NodeSelector) -> None: ...



from kubernetes_asyncio.client.models import V1ClientIPConfig
from typing import Any

class V1SessionAffinityConfig:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, client_ip: V1ClientIPConfig | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def client_ip(self) -> V1ClientIPConfig: ...
    @client_ip.setter
    def client_ip(self, client_ip: V1ClientIPConfig) -> None: ...



from kubernetes_asyncio.client.models import V2HPAScalingPolicy
from typing import Any

class V2HPAScalingRules:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, policies: list[V2HPAScalingPolicy] | None = None, select_policy: str | None = None, stabilization_window_seconds: int | None = None, tolerance: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def policies(self) -> list[V2HPAScalingPolicy]: ...
    @policies.setter
    def policies(self, policies: list[V2HPAScalingPolicy]) -> None: ...
    @property
    def select_policy(self) -> str: ...
    @select_policy.setter
    def select_policy(self, select_policy: str) -> None: ...
    @property
    def stabilization_window_seconds(self) -> int: ...
    @stabilization_window_seconds.setter
    def stabilization_window_seconds(self, stabilization_window_seconds: int) -> None: ...
    @property
    def tolerance(self) -> str: ...
    @tolerance.setter
    def tolerance(self, tolerance: str) -> None: ...



from kubernetes_asyncio.client.models import V1LabelSelector
from kubernetes_asyncio.client.models import V1PersistentVolumeClaim
from kubernetes_asyncio.client.models import V1PodTemplateSpec
from kubernetes_asyncio.client.models import V1StatefulSetOrdinals
from kubernetes_asyncio.client.models import V1StatefulSetPersistentVolumeClaimRetentionPolicy
from kubernetes_asyncio.client.models import V1StatefulSetUpdateStrategy
from typing import Any

class V1StatefulSetSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, min_ready_seconds: int | None = None, ordinals: V1StatefulSetOrdinals | None = None, persistent_volume_claim_retention_policy: V1StatefulSetPersistentVolumeClaimRetentionPolicy | None = None, pod_management_policy: str | None = None, replicas: int | None = None, revision_history_limit: int | None = None, selector: V1LabelSelector | None = None, service_name: str | None = None, template: V1PodTemplateSpec | None = None, update_strategy: V1StatefulSetUpdateStrategy | None = None, volume_claim_templates: list[V1PersistentVolumeClaim] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def min_ready_seconds(self) -> int: ...
    @min_ready_seconds.setter
    def min_ready_seconds(self, min_ready_seconds: int) -> None: ...
    @property
    def ordinals(self) -> V1StatefulSetOrdinals: ...
    @ordinals.setter
    def ordinals(self, ordinals: V1StatefulSetOrdinals) -> None: ...
    @property
    def persistent_volume_claim_retention_policy(self) -> V1StatefulSetPersistentVolumeClaimRetentionPolicy: ...
    @persistent_volume_claim_retention_policy.setter
    def persistent_volume_claim_retention_policy(self, persistent_volume_claim_retention_policy: V1StatefulSetPersistentVolumeClaimRetentionPolicy) -> None: ...
    @property
    def pod_management_policy(self) -> str: ...
    @pod_management_policy.setter
    def pod_management_policy(self, pod_management_policy: str) -> None: ...
    @property
    def replicas(self) -> int: ...
    @replicas.setter
    def replicas(self, replicas: int) -> None: ...
    @property
    def revision_history_limit(self) -> int: ...
    @revision_history_limit.setter
    def revision_history_limit(self, revision_history_limit: int) -> None: ...
    @property
    def selector(self) -> V1LabelSelector: ...
    @selector.setter
    def selector(self, selector: V1LabelSelector) -> None: ...
    @property
    def service_name(self) -> str: ...
    @service_name.setter
    def service_name(self, service_name: str) -> None: ...
    @property
    def template(self) -> V1PodTemplateSpec: ...
    @template.setter
    def template(self, template: V1PodTemplateSpec) -> None: ...
    @property
    def update_strategy(self) -> V1StatefulSetUpdateStrategy: ...
    @update_strategy.setter
    def update_strategy(self, update_strategy: V1StatefulSetUpdateStrategy) -> None: ...
    @property
    def volume_claim_templates(self) -> list[V1PersistentVolumeClaim]: ...
    @volume_claim_templates.setter
    def volume_claim_templates(self, volume_claim_templates: list[V1PersistentVolumeClaim]) -> None: ...



# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import sys
import traceback

from kubernetes_asyncio.client.rest import ApiException


def api_exception(e: ApiException) -> Exception:
    """
    Returns the proper Exception class for the given kubernetes.client.rest.ApiException object
    https://github.com/kubernetes/community/blob/master/contributors/devel/api-conventions.md#success-codes
    """
    _, _, exc_traceback = sys.exc_info()
    tb = "\n".join(traceback.format_tb(exc_traceback))
    return {
        400: BadRequestError,
        401: UnauthorizedError,
        403: ForbiddenError,
        404: NotFoundError,
        405: MethodNotAllowedError,
        409: ConflictError,
        410: GoneError,
        422: UnprocessibleEntityError,
        429: TooManyRequestsError,
        500: InternalServerError,
        503: ServiceUnavailableError,
        504: ServerTimeoutError,
    }.get(e.status, DynamicApiError)(e, tb)


class DynamicApiError(ApiException):
    """Generic API Error for the dynamic client"""

    def __init__(self, e: ApiException, tb=None) -> None:
        self.status = e.status
        self.reason = e.reason
        self.body = e.body
        self.headers = e.headers
        self.original_traceback = tb

    def __str__(self) -> str:
        error_message = [str(self.status), "Reason: {}".format(self.reason)]
        if self.headers:
            error_message.append("HTTP response headers: {}".format(self.headers))

        if self.body:
            error_message.append("HTTP response body: {!r}".format(self.body))

        if self.original_traceback:
            error_message.append(
                "Original traceback: \n{}".format(self.original_traceback)
            )

        return "\n".join(error_message)

    def summary(self) -> str:
        if self.body:
            if self.headers and self.headers.get("Content-Type") == "application/json":
                message = json.loads(self.body).get("message")
                if message:
                    return message

            return self.body.decode()
        else:
            return "{} Reason: {}".format(self.status, self.reason)


class ResourceNotFoundError(Exception):
    """Resource was not found in available APIs"""


class ResourceNotUniqueError(Exception):
    """Parameters given matched multiple API resources"""


class KubernetesValidateMissing(Exception):
    """kubernetes-validate is not installed"""


# HTTP Errors


class BadRequestError(DynamicApiError):
    """400: StatusBadRequest"""


class UnauthorizedError(DynamicApiError):
    """401: StatusUnauthorized"""


class ForbiddenError(DynamicApiError):
    """403: StatusForbidden"""


class NotFoundError(DynamicApiError):
    """404: StatusNotFound"""


class MethodNotAllowedError(DynamicApiError):
    """405: StatusMethodNotAllowed"""


class ConflictError(DynamicApiError):
    """409: StatusConflict"""


class GoneError(DynamicApiError):
    """410: StatusGone"""


class UnprocessibleEntityError(DynamicApiError):
    """422: StatusUnprocessibleEntity"""


class TooManyRequestsError(DynamicApiError):
    """429: StatusTooManyRequests"""


class InternalServerError(DynamicApiError):
    """500: StatusInternalServer"""


class ServiceUnavailableError(DynamicApiError):
    """503: StatusServiceUnavailable"""


class ServerTimeoutError(DynamicApiError):
    """504: StatusServerTimeout"""



from kubernetes_asyncio.client.api_client import ApiClient
from typing import Any
from typing import Awaitable

class LogsApi:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def log_file_handler(self, logpath: str, **kwargs: Any) -> Awaitable[None]: ...
    def log_file_handler_with_http_info(self, logpath: str, **kwargs: Any) -> Awaitable[None]: ...
    def log_file_list_handler(self, **kwargs: Any) -> Awaitable[None]: ...
    def log_file_list_handler_with_http_info(self, **kwargs: Any) -> Awaitable[None]: ...



from kubernetes_asyncio.client.models import V1IPBlock
from kubernetes_asyncio.client.models import V1LabelSelector
from typing import Any

class V1NetworkPolicyPeer:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, ip_block: V1IPBlock | None = None, namespace_selector: V1LabelSelector | None = None, pod_selector: V1LabelSelector | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def ip_block(self) -> V1IPBlock: ...
    @ip_block.setter
    def ip_block(self, ip_block: V1IPBlock) -> None: ...
    @property
    def namespace_selector(self) -> V1LabelSelector: ...
    @namespace_selector.setter
    def namespace_selector(self, namespace_selector: V1LabelSelector) -> None: ...
    @property
    def pod_selector(self) -> V1LabelSelector: ...
    @pod_selector.setter
    def pod_selector(self, pod_selector: V1LabelSelector) -> None: ...



from kubernetes_asyncio.client.models import V1LabelSelector
from kubernetes_asyncio.client.models import V1NetworkPolicyEgressRule
from kubernetes_asyncio.client.models import V1NetworkPolicyIngressRule
from typing import Any

class V1NetworkPolicySpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, egress: list[V1NetworkPolicyEgressRule] | None = None, ingress: list[V1NetworkPolicyIngressRule] | None = None, pod_selector: V1LabelSelector | None = None, policy_types: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def egress(self) -> list[V1NetworkPolicyEgressRule]: ...
    @egress.setter
    def egress(self, egress: list[V1NetworkPolicyEgressRule]) -> None: ...
    @property
    def ingress(self) -> list[V1NetworkPolicyIngressRule]: ...
    @ingress.setter
    def ingress(self, ingress: list[V1NetworkPolicyIngressRule]) -> None: ...
    @property
    def pod_selector(self) -> V1LabelSelector: ...
    @pod_selector.setter
    def pod_selector(self, pod_selector: V1LabelSelector) -> None: ...
    @property
    def policy_types(self) -> list[str]: ...
    @policy_types.setter
    def policy_types(self, policy_types: list[str]) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1alpha3DeviceTaintRule
from kubernetes_asyncio.client.models import V1alpha3DeviceTaintRuleList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class ResourceV1alpha3Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_device_taint_rule(self, body: V1alpha3DeviceTaintRule, **kwargs: Any) -> Awaitable[V1alpha3DeviceTaintRule]: ...
    def create_device_taint_rule_with_http_info(self, body: V1alpha3DeviceTaintRule, **kwargs: Any) -> Awaitable[tuple[V1alpha3DeviceTaintRule, int, CIMultiDictProxy]]: ...
    def delete_collection_device_taint_rule(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_device_taint_rule_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_device_taint_rule(self, name: str, **kwargs: Any) -> Awaitable[V1alpha3DeviceTaintRule]: ...
    def delete_device_taint_rule_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1alpha3DeviceTaintRule, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_device_taint_rule(self, **kwargs: Any) -> Awaitable[V1alpha3DeviceTaintRuleList]: ...
    def list_device_taint_rule_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1alpha3DeviceTaintRuleList, int, CIMultiDictProxy]]: ...
    def patch_device_taint_rule(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1alpha3DeviceTaintRule]: ...
    def patch_device_taint_rule_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1alpha3DeviceTaintRule, int, CIMultiDictProxy]]: ...
    def read_device_taint_rule(self, name: str, **kwargs: Any) -> Awaitable[V1alpha3DeviceTaintRule]: ...
    def read_device_taint_rule_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1alpha3DeviceTaintRule, int, CIMultiDictProxy]]: ...
    def replace_device_taint_rule(self, name: str, body: V1alpha3DeviceTaintRule, **kwargs: Any) -> Awaitable[V1alpha3DeviceTaintRule]: ...
    def replace_device_taint_rule_with_http_info(self, name: str, body: V1alpha3DeviceTaintRule, **kwargs: Any) -> Awaitable[tuple[V1alpha3DeviceTaintRule, int, CIMultiDictProxy]]: ...



from typing import Any

class V1PodCertificateProjection:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, certificate_chain_path: str | None = None, credential_bundle_path: str | None = None, key_path: str | None = None, key_type: str | None = None, max_expiration_seconds: int | None = None, signer_name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def certificate_chain_path(self) -> str: ...
    @certificate_chain_path.setter
    def certificate_chain_path(self, certificate_chain_path: str) -> None: ...
    @property
    def credential_bundle_path(self) -> str: ...
    @credential_bundle_path.setter
    def credential_bundle_path(self, credential_bundle_path: str) -> None: ...
    @property
    def key_path(self) -> str: ...
    @key_path.setter
    def key_path(self, key_path: str) -> None: ...
    @property
    def key_type(self) -> str: ...
    @key_type.setter
    def key_type(self, key_type: str) -> None: ...
    @property
    def max_expiration_seconds(self) -> int: ...
    @max_expiration_seconds.setter
    def max_expiration_seconds(self, max_expiration_seconds: int) -> None: ...
    @property
    def signer_name(self) -> str: ...
    @signer_name.setter
    def signer_name(self, signer_name: str) -> None: ...
